package com.unimas.esql_api.v2;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.io.File;
import java.io.InputStream;
import java.io.IOException;

import com.google.gson.JsonObject;
import org.apache.http.Header;
import org.apache.http.HeaderElement;
import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.codec.binary.Base64;

import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonParser;
import com.google.gson.JsonElement;


public class ESql {
    private  Gson gson;
    private JsonParser parser;

    private String server;
    private String username = "";
    private String password = "";
    private String cookie = "";
    private String current_sql = "";

    // 接口返回结构
    public class Result {
        public String state;
        public int code;
        public String message = "";
        public int took = 0;
        public int cost = 0;
        public long count = 0;
        public JsonArray data;
        // 文件接口专用
        public byte[] file;
    }

    public ESql(String server_host) {
        gson = new Gson();
        parser = new JsonParser();
        server = server_host;
    }

    public ESql(String server_host, String username, String password) {
        this(server_host);
        this.username = username;
        this.password = password;
    }

    private void setCookie(Header[] headers) {
        for (Header header : headers) {
            if (header != null) {
                HeaderElement[] headerElements = header.getElements();
                if (headerElements != null) {
                    for (HeaderElement headerElement : headerElements) {
                        if ("session".equals(headerElement.getName()))
                            this.cookie += "session=" + headerElement.getValue();
                    }
                }
            }
        }
    }

    private ESql.Result post(String url) {
        List<NameValuePair> params = new ArrayList<>();
        return post(url, params, false, false);
    }

    private ESql.Result post(String url, List<NameValuePair> params) {
        return post(url, params, false, false);
    }

    private ESql.Result post(String url, List<NameValuePair> params, Boolean login, Boolean file_api) {
        String error = "";
        ESql.Result result = new ESql.Result();
        try {
            URI uri = new URIBuilder().setScheme("http").setHost(server).setPath(url).build();
            HttpPost httpPost = new HttpPost(uri);
            httpPost.setEntity(new UrlEncodedFormEntity(params, HTTP.UTF_8));
            httpPost.addHeader("Cookie", cookie);
            CloseableHttpClient httpclient= HttpClients.createDefault();
            CloseableHttpResponse response = httpclient.execute(httpPost);
            try {
                HttpEntity entity = response.getEntity();
                if (file_api) {
                    String content_type = entity.getContentType().getValue();
                    if (! content_type.startsWith("application/json")) {
                        int numRead;
                        int offset = 0;
                        result.code = 0;
                        result.state = "Success";
                        Long l = entity.getContentLength();
                        Integer length = l.intValue();
                        result.file = new byte[length];
                        InputStream content = entity.getContent();

                        while (offset < length && (numRead = content.read(result.file, offset, length - offset)) >= 0) {
                            offset += numRead;
                        }
                        // 确保所有数据均被读取
                        if (offset != length) {
                            throw new IOException("Could not completely read file ");
                        }
                        response.close();
                        httpclient.close();
                        return result;
                    }
                }
                if (entity != null) {
                    String entityStr = EntityUtils.toString(entity);
//                    System.out.println(entityStr);
                    JsonObject json = parser.parse(entityStr).getAsJsonObject();
                    result.code = json.get("code").getAsInt();
                    result.state = json.get("state").getAsString();
                    result.count = (json.get("count") == null) ? 0 : json.get("count").getAsInt();
                    result.cost = (json.get("cost") == null) ? 0 : json.get("cost").getAsInt();
                    result.took = (json.get("took") == null) ? 0 : json.get("took").getAsInt();
                    result.message = (json.get("message") == null) ? "" : json.get("message").getAsString();
                    result.data = (json.get("data") == null) ? new JsonArray() : json.get("data").getAsJsonArray();
                }
                if (login) {
                    setCookie(response.getHeaders("Set-Cookie"));
                }
            } catch (Exception e) {
                error = e.getMessage();
            }
            response.close();
            httpclient.close();
        } catch (Exception e) {
            error = e.getMessage();
        }
        if (!error.equals("")) {
            result.state = "CanNotAccess";
            result.code = 401;
            result.message = "can't access server [" + server + url + "] " + error;
        }
        return result;
    }

    public ESql.Result login() {
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("username", username));
        params.add(new BasicNameValuePair("password", password));
        return post("/login2", params, true, false);
    }

    public ESql.Result execute(String sql) {
        current_sql = sql;
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("sql", sql));
        return post("/sql", params);
    }

    public ESql.Result logout() {
        return post("/logout2");
    }

    public String fileSet(String file_path) throws IOException {
        return fileSet(file_path, new HashMap());
    }

    public String fileSet(String file_path, Map params) throws IOException {

        File file_obj = new File(file_path.trim());
        byte[] content = FileUtils.readFileToByteArray(new File(file_path));
        byte[] base64_content = Base64.encodeBase64(content);

        Map file_set = new HashMap();
        file_set.put("_name", file_obj.getName());
        file_set.put("_content", new String(base64_content));
        file_set.put("_content_length", content.length);

        file_set.putAll(params);    // "_content_type"  "_language"

        return "'" + gson.toJson(file_set) + "'";
    }

    public ESql.Result fileGet(String file_id) {
        List<NameValuePair> params = new ArrayList<>();
        params.add(new BasicNameValuePair("file_id", file_id));
        return post("/file_get", params, false, true);
    }

    // 调试用
    public void print_result(ESql.Result result) {
        if (! current_sql.equals("")) {
            System.out.println(current_sql + "\n");
            current_sql = "";
        }
        System.out.println("-- " + result.state + "[" + result.code + "]  count: " + result.count +
                " cost: " + result.cost + " took: " + result.took + " message: " + result.message);
        if (result.count > 0) {

            for (int i = 0; i < result.count; i++) {
                if (result.data.get(i).isJsonObject()) {
                    JsonObject row = result.data.get(i).getAsJsonObject();
                    System.out.print("-- ");
                    for (Map.Entry<String, JsonElement> item : row.entrySet()) {
                        System.out.print(item.getKey() + ": " + item.getValue().toString() + "\t\t");
                    }
                    System.out.println("");
                }
            }
            System.out.println("------------------------------------------------");
        }
        System.out.println("--------------------------------------------------------------------------\n");
    }
}
